
#ifndef VKBASEDATA_H
#define VKBASEDATA_H

#include <TFile.h>
#include <TTree.h>
#include <TRandom3.h>
#include <TF1.h>

#include <string>


struct VKEvent {
    Float_t ES; // Energy Signal
    Float_t EV; // Energy Veto
    UInt_t  PS; // Photons Signal
    UInt_t  PV; // Photons Veto

    Float_t EF2; // Energy Front v2.0
    Float_t ES2; // Energy Signal v2.0
    Float_t EV2; // Energy Veto v2.0
};


// base class (which provides some implementation)
class VKBaseData {

    public:
        VKBaseData();
        virtual ~VKBaseData();
    
        void CreateOutput(std::string filename);

        virtual size_t GetEntries() const = 0;
        virtual double GetNormalization() const = 0;
        virtual double GetArea() const = 0;
        virtual const VKEvent& GetEvent(size_t id) = 0;
        virtual bool HasV2Branches() const = 0;
        
        inline void AddEnergyResolution(bool enable) {fAddRes=enable;}

    protected:

    private:
        void SampleResolution(Float_t& e);
        void CreateOutputBranches(TTree * t);
        
        TF1 * fRes;
        TRandom3 * fRNG;

        bool fAddRes;
        double fP0, fP1, fTauRel;
        
        TFile *fOutputFile;
        TTree *fOutputTree;
        Float_t _ESout;
        Float_t _EVout;
        UInt_t  _PSout;
        UInt_t  _PVout;
        Float_t _EF2out;
        Float_t _ES2out;
        Float_t _EV2out;
};

#endif
