//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "globals.hh"

#include "VKLeadBlock.hh"
#include "VKUtil.hh"

#include "G4LogicalSkinSurface.hh"
#include "G4LogicalBorderSurface.hh"
#include "G4SystemOfUnits.hh"
#include "G4Cons.hh"
#include "TMath.h"


VKLeadBlock::VKLeadBlock(G4RotationMatrix *pRot,
                             const G4ThreeVector &tlate,
                             G4LogicalVolume *pMother_log,
                             G4bool pMany,
                             G4int pCopyNo,
                             const VKDetectorConstruction* pDetConstr)

 // Pass info to the G4PVPlacement constructor
 : G4PVPlacement(pRot, 
                 tlate,
                 CreateTemporaryLogicalVolume(), // <-- see VKUtil.cpp
                 "leadBlock", 
                 pMother_log, 
                 pMany, 
                 pCopyNo),
   fDetConstr(pDetConstr),              
   fMother_log(pMother_log)
{

    // default dimensions    
    SetDefaults();

    // List of materials
    fMaterials = VKMaterials::GetInstance();

    // Get materials
    G4Material* lead  = fMaterials -> GetMaterial("LEAD");
    assert(lead != nullptr);
        
    // solid
    fLeadBlock_sol = new G4Box("leadBlock_sol", fXDim/2, fYDim/2, fZDim/2);

    // logical
    fLeadBlock_log = new G4LogicalVolume(fLeadBlock_sol, lead, "leadBlock_log");

    // physical  
//    new G4PVPlacement(0, G4ThreeVector(fXPos, fYPos, fZPos), fLeadBlock_log, 
//                      "leadBlock", fMother_log, false, 0, fCheckOverlaps);

    // visualization attributes
    VisAttributes();    

    // Set the logical volume
    this -> SetLogicalVolume(fLeadBlock_log); // method inherited from G4VPhysicalVolume
    
    // Set position
    this -> SetTranslation(G4ThreeVector(fXPos, fYPos, fZPos));
    
    // Check for overlaps
    if (fCheckOverlaps) this -> CheckOverlaps();
}


void VKLeadBlock::SetDefaults(){

    fCheckOverlaps = fDetConstr -> GetCheckOverlaps();

    // dimensions
    fXDim = 10*cm;
    fYDim =  5*cm;
    fZDim = 10*cm;
    
    // position
    fXPos =  0.0*cm;
    fYPos =  7.5*cm;
    fZPos = 32.0*cm;
}


void VKLeadBlock::VisAttributes() {

    // Alu frame
    G4VisAttributes* leadBlock_va = new G4VisAttributes(G4Colour(0.5, 0.0, 0.5)); // purple
    leadBlock_va   -> SetVisibility(true);
    leadBlock_va   -> SetForceWireframe(false);
    fLeadBlock_log -> SetVisAttributes(leadBlock_va);
}
